
      subroutine grerik(f,l,x,y)
c     ==================biodata ftn4
c     ==================smhi ftn10r1
c     ==================smhi 820226
c
c purp  grerik converts greenwich-coordinates into numerical rak
c
c call grerik(f,l,x,y)

c
c args input
c      f    latitude (decimal degrees)
c      l    longitude (decimal degrees)
c      output
c      x    plane coordinate south-north (m)
c      y    plane coordinate west-east (m)
c
c subr filaxy
c
      implicit real(a-z)
      dimension beta(3),alfa(3)
      data a/6366742.5194/,lamno/0.27590649629/,
     *     ynoll/1500000./,pi/3.1415926536/,
     *     beta/835225270.,756302.,1193./,
     *     alfa/-3346494763.,466475.,-8121./
c
      fi=f*pi/180.
      lambda=l*pi/180.
c
c     h[r kan man l[gga in en entrypoint
c     om man har koordinater uttryckta i radianer
c
      call filaxy(fi,lambda,ynoll,lamno,a,alfa,beta,x,y)
      return
      end






      subroutine greutm(f,l,lamno,x,y)
c     ==================biodata ftn4
c     ==================smhi 860905
c
c purp  greutm converts greenwich-coordinates to numerical utm
c
c call greutm(f,l,lamno,x,y)

c
c args input
c      f      latitude (decimal degrees)
c      l      longitude (decimal degrees)
c      lamno  longitude of central meridian (rad)
c      output
c      x      south-north coordinate (m)
c      y      west-east coordinate (m)
c
c subr filaxy
c
      implicit real(a-z)
      dimension alfa(3),beta(3)
      data a/6365107.458258/,pi/3.1415926536/,
     *     alfa/-3370779072.,4734448.,-8299./,
     *     beta/841275989.,767304.,1219./
c
      fi=f*pi/180.
      lambda=l*pi/180.
c
c     entrypoint om man har koordinater
c     uttryckta i radianer
c
      call filaxy(fi,lambda,500000.,lamno,a,alfa,beta,x,y)
      return
      end


      subroutine rikgre(x,y,f,l)
c     ==================biodata ftn4
c     ==================smhi 860522
c
c purp  rikgre converts  numerical rak to greenwich-coordinates
c
c call rikgre(x,y,f,l)

c
c args input
c      x    south-north coordinate (m)
c      y    west-east coordinate (m)
c      output
c      f    latitude (decimal degrees)
c      l    longitude (decimal degrees)
c
c subr xyfila
c
      implicit real(a-z)
      dimension gamma(3),delta(3)
      data a/6366742.5194/,pi/3.1415926536/,
     *     delta/835225613.,58706.,166./,
     *     gamma/3346491640.,6532540.,17488./,
     *     lamno/0.27590649629/
c
      call xyfila(x,y,1500000.,lamno,a,gamma,delta,fi,lambda)
      l=lambda*180./pi
      f=fi*180./pi
      return
      end


      subroutine utmgre(x,y,lamno,fi,lambda)
c     ==================biodata ftn4
c     ==================smhi 860905
c
c purp  utmgre converts  numeric utm to greenwich-coordinates
c
c call rikgre(x,y,lamno,fi,lambda)

c
c args input
c      x      plane coordinate south-north (m)
c      y      plane coordinate west-east (m)
c      lamno  longitude of central meridian (rad)
c      output
c      fi     latitude (decimal degrees)
c      lambda longitude (decimal degrees)
c
c subr xyfila
c
c     rutinen klarar bara norra halvklotet
c     enkelt att [ndra s] att [ven s\dra delen kommer med
c     om n]gon plockar fram hur notationen ser ut d[r
c
      implicit real(a-z)
      dimension delta(3),utm(2),gamma(3)
      integer utm
      data a/6365107.458258/,pi/3.1415926536/,
     *     delta/841276339.,59564.,170./,
     *     gamma/3370775881.,6627691.,17872./
c
      call xyfila(x,y,500000.,lamno,a,gamma,delta,fi,lambda)
      lambda=lambda*180./pi
      fi=fi*180./pi
      return
      end


      subroutine filaxy(fi,lambda,ynoll,lamno,a,alfa,beta,x,y)
c     ==================biodata ftn4
c     ==================smhi ftn10r1
c     ==================smhi 820226
c
c purp filaxy converts greenwich to plane coord in gauss conform proj
c
c call filaxy(fi,lambda,ynoll,lamno,a,alfa,beta,x,y)
c
c args input
c      fi      latitude (rad)
c      lambda  longitude (rad)
c      ynoll   y-value for central merdian
c      lamnoll longitude of central meridian (rad)
c      a       constants to determine the
c      alfa    ellipsoid of rotation which
c      beta    approximates the earth
c      output
c      x       plane south-north coordinate (meter)
c      y       plane west-east coordinate (meter)
c
c subr functions funq1 and funq2
c      funq3-funq6 as external functions
c
      implicit real (a-z)
      external funq3,funq4,funq5,funq6
      dimension alfa(3),beta(3)
c
      b=fi+funq1(fi,alfa)
      xi=atan(tan(b)/cos(lambda-lamno))
      et=cos(b)*sin(lambda-lamno)
      eta=alog((1.+et)/(1.-et))/2.
      x=a*(xi+funq2(xi,eta,beta,funq3,funq6))
      y=ynoll+a*(eta+funq2(xi,eta,beta,funq4,funq5))
      return
      end


      subroutine xyfila(x,y,ynoll,lamno,a,gamma,delta,fi,lambda)
c     ==================biodata ftn4
c     ==================smhi 860522
c
c purp xyfila converts coordinates in gauss conform proj to greenwich
c
c call xyfila(x,y,ynoll,lamnoll,a,gamma,delta,fi,lambda)
c
c args input
c      x       south-north coordinate (meter)
c      y       west-east coordinate (meter)
c      ynoll   y-value for central merdian
c      lamnoll longitude of central meridian (rad)
c      a       constants to determine the
c      gamma    ellipsoid of rotation which
c      delta    approximates the earth
c      output
c      fi      latitude (rad)
c      lambda  longitude (rad)
c
c subr functions funq1 and funq2
c      funq3-funq6 as external functions
c
      implicit real (a-z)
      external funq3,funq4,funq5,funq6
      dimension gamma(3),delta(3)
c
      xi=x/a
      eta=(y-ynoll)/a
      xiprim=xi-funq2(xi,eta,delta,funq3,funq6)
      etaprim=eta-funq2(xi,eta,delta,funq4,funq5)
      b=asin(sin(xiprim)/cosh(etaprim))
      lambda=lamno+atan(sinh(etaprim)/cos(xiprim))
      fi=b+funq1(b,gamma)
      return
      end


      function funq1(x,c)
c     ==============biodata ftn4
c     ==============smhi ftn10r1
c     ==============smhi 820226
c
c purp funq1 is used by filaxy and xyfila
c
      dimension c(3)
      funq1=c(1)*sin(2.*x)+c(2)*sin(4.*x)+c(3)*sin(6.*x)
      funq1=funq1*1.e-12
      return
      end


      function funq2(x,y,z,f1,f2)
c     ===============biodata ftn4
c     ===============smhi ftn10r1
c     ===============smhi 820226
c
c purp funq2 is used by filaxy and xyfila
c
      dimension z(3)
c
      funq2=z(1)*f1(2.*x)*f2(2.*y)+z(2)*f1(4.*x)*f2(4.*y)+
     *      z(3)*f1(6.*x)*f2(6.*y)
      funq2=funq2*1.e-12
      return
      end


      function funq3(x)
c     ===============biodata ftn4
c     ===============smhi ftn10r1
c     ===============smhi 820226
c
c purp funq3 is used as external function by filaxy and xyfila
c
      funq3=sin(x)
      return
      end


      function funq4(x)
c     ===============biodata ftn4
c     ===============smhi ftn10r1
c     ===============smhi 820226
c
c purp funq4 is used as external function by filaxy and xyfila
c
      funq4=cos(x)
      return
      end


      function funq5(x)
c     ===============biodata ftn4
c     ===============smhi ftn10r1
c     ===============smhi 820226
c
c purp funq5 is used as external function by filaxy and xyfila
c
      funq5=sinh(x)
      return
      end


      function funq6(x)
c     ===============biodata ftn4
c     ===============smhi ftn10r1
c     ===============smhi 820226
c
c purp funq6 is used as external function by filaxy and xyfila
c
      funq6=cosh(x)
      return
      end

