      subroutine oi(mobs,nbox,nmax,nmin,npar,iy,im,id,ih,exper,
     2           idim,jdim,gscale,rmax,sdevfg,sdevob,
     3           tolera1,tolera2,
     4           lunobs,lunfgs,lunana,
     5           latm,lonm,dlat,dlon,clat,lfgs)
c     +-----------------------------------------------------------------+
c     + A simple analysis program for OI                                +
c     +  1. variable declaration                                        +
c     +     1.1  dimension                                              +
c     +     1.2  namelist                                               +
c     +     1.3  grid paramters                                         +
c     +     1.4  observational data fields                              +
c     +     1.5  observations selected for analysis                     +
c     +     1.6  information about rejected data                        +
c     +     1.7  covariance matrix fields                               +
c     +     1.8  working arrays                                         +
c     +     1.9  administrative parameters                              +
c     +  2. derived parameters and check                                +
c     +  3. write some input parameters                                 +
c     +  4. open input and output files                                 +
c     +     4.1 observation file                                        +
c     +     4.2 firstguess file                                         +
c     +     4.3 result file                                             +
c     +     4.4 check file                                              +
c     +  5. read and (gross) check observations                         +
c     +  6. analysis loop over all parameters                           +
c     +     6.1 read firstguess                                         +
c     +     6.2 OI analysis                                             +
c     +         --- loop for box's [1 or 9 (latter with fixed dims)]--  +
c     +         6.2.1 select influencing observations                   +
c     +         6.2.2 construct and convert covariance matrix           +
c     +         6.2.3 check obs (bad ob is only flagged ??)             +
c     +         6.2.4 transform fgs to observation locations            +
c     +         6.2.5 analysis of parameter at position (lat,lon)       +
c     +         6.2.6 restore analysis data                             +
c     +         ----- end of loop for box's --------------------------  +
c     +         6.2.7 add all values from related boxes                 +
c     +  7. derived variables                                           +
c     +  8. statistics                                                  +
c     +  9. store analyses and observations                             +
c     + 10. close obs, ana and chk files                                +
c     +     end of the analysis                                         +
c     + A1. write format                                                +
c     +-----------------------------------------------------------------+
c 1   ============================================ variable declaration =
      implicit none
c 1.1 ------------------------------------------------------- dimension -
      integer mobs,npar,idim,jdim,nmin,nmax,nhor,nho2
c 1.2 -------------------------------------------------------- namelist -
      integer   nbox,iy,im,id,ih
      real      latm,lonm,dlat,dlon,rmax
      real      gscale(npar),sdevfg(npar),sdevob(npar)
      real      tolera1(npar),tolera2(npar)
      character exper*3
      logical 	lfgs
c 1.3 ------------------------------------------------------------ grid -
      integer iana,jana
      real    lats,lonw,dlatfgs,dlonfgs
      real    ana(idim,jdim),fgs(idim,jdim)
      real    p_ana(idim,jdim),t_ana(idim,jdim),u_ana(idim,jdim)
      real    v_ana(idim,jdim),d_ana(idim,jdim)
      real    zana(idim,jdim)
      real    grad_o(idim,jdim),grad_b(idim,jdim),grad(idim,jdim)
      real    zgrad_o(idim,jdim)
      real    box(11,11,3,3) ! for nbox=9
c 1.4 --------------------------------------- observational data fields -
      integer   nobs,iiiobs(mobs),nrobs(mobs),flgobs(mobs,npar)
      real      oriobs(mobs,npar),latobs(mobs),lonobs(mobs)
      character filnm*12
c 1.5 ------------------------------ observations selected for analysis -
      integer nana,iiival(nmax),nrval(nmax)
      real    latval(nmax),lonval(nmax),valobs(nmax),valmean
      real    valfgs(nmax),valinn(nmax),valana(nmax)
      real    valinb(nmax)
c 1.6 --------------------------------- information about rejected data -
      integer nskip,lskip(nmax),flag(nmax),dev(nmax)
      logical skip(nmax)
c 1.7 ---------------------------------------- covariance matrix fields -
      real co(nmax*nmax),coi(nmax*nmax),coired(nmax*nmax)
c 1.8 -------------------------------------------------- working arrays -
      real    w1(nmax*nmax),w2(nmax*nmax),w3(nmax*nmax),w4(nmax*nmax)
      real    lat,lon
      integer i,j,k,i1,i2,ibox,jbox
c 1.9  -------------------------------------- administrative parameters -
      integer iloop,jloop,ipar,ios
      integer lunobs,lunfgs,lunana
      integer nro(3,3)
      real    clat
c 2   ================================== dereived parameters and check =
      nhor=idim*jdim
      nho2=nhor*nhor
      if(nbox.eq.9) then
	 if(lfgs) stop ' The 9-box setup has no fgs'
	 if(idim.ne.21 .or. jdim.ne.21)
     +            stop ' The 9-box setup is hard-coded for 21x21 '
	 ibox = 3
	 jbox = 3
      else if(nbox.eq.1) then
	 ibox = 1
	 jbox = 1
      else
	 write(6,*) 'The number of analysis box can only be 1 or 9'
	 stop
      endif
c 3   ===================================== write some input parameters =
      write(6,1001)
      write(6,'(2a)')' exp : ',exper
      write(6,'(a,4(1x,i2.2))')' date:',iy-1900,im,id,ih
      write(6,1001)
      write(6,*)' parameter              pressure       temperature'
      write(6,1002)
      write(6,*)' sdev of fg          ',sdevfg
      write(6,*)' sdev of obs         ',sdevob
      write(6,*)' tolerance 1         ',tolera1
      write(6,*)' tolerance 2         ',tolera2
      write(6,*)' Gaussian parameters ',gscale
      write(6,1001)
c 4   ===================================== open input and output files =
c 4.1 ------------------------------------------------ observation file -
      write(filnm(1:12),'(4i2.2,a4)') mod(iy,100),im,id,ih,'.obs'
      open(lunobs,file=filnm,iostat=ios,status='old')
c 4.2 ------------------------------------------------- firstguess file -
      if(lfgs) then
         write(filnm(1:12),'(4i2.2,a4)') mod(iy,100),im,id,ih,'.fgs'
         open(lunfgs,file=filnm,iostat=ios,status='old')
         read(lunfgs,*) iana,jana,lats,lonw,dlatfgs,dlonfgs
         if(dlatfgs.ne.dlat.or.dlonfgs.ne.dlon) then
            write(6,*) 'from fgs, dlon and dlat =',dlonfgs,dlatfgs
            write(6,*) 'namelist, dlon and dlat =',dlon,dlat
            stop
         endif
      endif
c 4.3 ----------------------------------------------------- result file -
      write(filnm(1:12),'(4i2.2,a,a3)')mod(iy,100),im,id,ih,'.',exper
      open(lunana,file=filnm,iostat=ios,status='unknown')
c 5   ============================= read and (gross) check observations =
      call getobs(lunobs,npar,mobs,nobs,
     +            iiiobs,nrobs,latobs,lonobs,oriobs,flgobs)
c 6   =============================== analysis loop over all parameters =
      do ipar=1,2
c 6.1    ---------------------------------------------- read firstguess -
         if(lfgs) call getfld(idim,jdim,fgs,lunfgs)
c 6.2    -------------------------------------------------- OI analysis -
c        ----------------------------------------- loops over all box's -
         do 626 jloop=1,jbox
	    if(nbox.eq.9) then
               lat=latm+(jloop-2)*5.*dlat
	    else
               lat=latm
	    endif
         do 626 iloop=1,ibox
	    if(nbox.eq.9) then
               lon=lonm+(iloop-2)*5.*dlon
	    else
               lon=lonm
	    endif
c 6.2.1     ........................... select influencing observations .
            call choose_box(nbox,idim,jdim,0,0,
     +            iiiobs,latobs,lonobs,oriobs(1,ipar),
     +            nrobs,nobs,lat,lon,dlat,dlon,rmax,nmax,nmin,
     +            iiival,nrval,latval,lonval,valobs,nana)
            nro(iloop,jloop)=nana
c 6.2.2     .................... construct and invert covariance matrix .
            call covmat(lat,clat,sdevfg(ipar),sdevob(ipar),
     +                  gscale(ipar),nana,latval,lonval,co)
            call linds(nana,co,nana,coi,nana,w4)
c 6.2.3     .......... check observations, reduce coi for rejected data .
            call check(nana,co,coi,coired,valobs,nskip,lskip,skip,
     +                 dev,flag,w1,w2,w3,w4,
     +	               sdevob(ipar),sdevfg(ipar),tolera1(ipar),
     +	               tolera2(ipar),valmean,iiival)
            do k=1,nana
	       if( skip(k) ) 
     +            write(6,'(a,i1,a,f10.2,a,i6,a,f5.1,a,f5.1,a)')  
     +            ' par=',ipar,' with value=',valobs(k),
     +            ' is rejected at station',iiival(k),
     +            ' (',latval(k),',',lonval(k),')'
            enddo
c 6.2.4     ................................... convert fgs to obs-grid .
            if(lfgs) then
	       call H(idim,jdim,nana,latval,lonval,
     +                lat,lon,dlat,dlon,fgs,valfgs)
	    endif
c 6.2.5     ............... analysis of parameter at position (lat,lon) .
            call oibox(ipar,nbox,lfgs,
     +                  idim,jdim,ana,fgs,
     +                  lat,lon,dlat,dlon,clat,nana,latval,lonval,
     +                  valmean,valobs,skip,valfgs,w1,coired,
     +                  sdevfg(ipar),gscale(ipar),rmax)
	    if(nbox.eq.9) then
	       do i=1,11
	       do j=1,11
	          box(i,j,iloop,jloop) = ana(i,j)
	       enddo
	       enddo
	    else
	       if(ipar.eq.1) then
		  call hcpa2b(idim*jdim,ana,p_ana)
	       else
		  call hcpa2b(idim*jdim,ana,t_ana)
	       endif
	    endif
c 6.2.6     .................................... restore analysis data  .
            call saveres(mobs,npar,nana,ipar,nrval,flgobs,flag)
  626    continue
         write(6,'(3x,a,2x,3i5,/,23x,3i5,/,23x,3i5)') 
     +      ' nobs in each box:',((nro(i1,i2),i1=1,ibox),i2=jbox,1,-1)
         write(6,1001)
c 6.2.7     ....................... add all values from related boxes .
  627    continue
	 if(nbox.eq.9) then
            if(ipar.eq.2) then
               call addbox(box,idim,jdim,t_ana)
	    else
               call addbox(box,idim,jdim,p_ana)
            endif
	 endif
      enddo
c 7   ============================================== derived variables =
      call derivf(t_ana,p_ana,
     +            u_ana,v_ana,d_ana,idim,jdim,latm,clat,dlat)
c 8   ===================================================== statistics =
	 write(6,*)' statistics :'
	 call statis('   p min/max = ',idim,jdim,p_ana)
	 call statis('   t min/max = ',idim,jdim,t_ana)
	 call statis('   u min/max = ',idim,jdim,u_ana)
	 call statis('   v min/max = ',idim,jdim,v_ana)
	 call statis(' dyn min/max = ',idim,jdim,d_ana)
      write(6,1001)
c  9  =========================== store the analyses and observations =
      write(lunana,'(2i4,4f10.4)')idim,jdim,
     +              latm-jdim/2*dlat,lonm-idim/2*dlon,dlat,dlon
      call putfld('p',lunana,idim,jdim,p_ana,idim,jdim)
      call putfld('t',lunana,idim,jdim,t_ana,idim,jdim)
cc    call putfld('u',lunana,idim,jdim,u_ana,idim,jdim)
cc    call putfld('v',lunana,idim,jdim,v_ana,idim,jdim)
      call putobs('obs',mobs,npar,nobs,lunana,
     x		iiiobs,latobs,lonobs,oriobs,flgobs)
c 10  ================================== close obs, ana and chk files =
      close(lunobs)
      close(lunfgs)
      close(lunana)
c     =========================================== end of the analysis =
      write(6,*)'analysis finished without any error!'
      write(6,1001)
      return
c A1  ======================================================= formats =
1001  format("=======================================================")
1002  format("-------------------------------------------------------")
6211  format(1x,a,20(/,2x,10i7))
6212  format(1x,a,20(/,2x,10f7.1))
      end
