      subroutine var(mobs,npar,iy,im,id,ih,exper,
     2           idim,jdim,gscale,rmax,sdevfg,sdevob,
     3           lunobs,lunfgs,lunana,lunref,
     4           latm,lonm,dlat,dlon,clat,
     5           lfgs,lref,maxiter,alpha)
c     +-----------------------------------------------------------------+
c     + A simple analysis program for VAR                               +
c     +  1. variable declaration                                        +
c     +     1.1  dimension                                              +
c     +     1.2  namelist                                               +
c     +     1.3  grid paramters                                         +
c     +     1.4  observational data fields                              +
c     +     1.5  observations selected for analysis                     +
c     +     1.6  information about rejected data                        +
c     +     1.7  covariance matrix fields                               +
c     +     1.8  administrative parameters                              +
c     +  2. derived parameters                                          +
c     +  3. read and write input parameters (namelist)                  +
c     +  4. open input and output files                                 +
c     +     4.1 observation file and the reference file (BLUE)          +
c     +     4.2 firstguess file                                         +
c     +     4.3 result file                                             +
c     +     4.4 check file                                              +
c     +  5. read and (gross) check observations                         +
c     +  6. analysis loop over all parameters                           +
c     +     6.1 read firstguess                                         +
c     +     6.2 VAR analysis                                            +
c     +         6.2.1 select influencing observations                   +
c     +         6.2.2 construct and invert  covariance matrix           +
c     +         6.2.3 use fgs as ana before at iter=0                   +
c     +         --- loop for minimization iterations -----------------  +
c     +         6.2.4 transform fgs to observation locations            +
c     +         6.2.5 calculation innovation vector                     +
c     +         6.2.6 cost function gradient for observation grad(Jo)   +
c     +         6.2.6 cost function gradient for background  grad(Jb)   +
c     +         6.2.8 cost function gradient grad=grad(Jo)+grad(Jb)     +
c     +         6.2.9 minimization step                                 +
c     +         ----- end of loop for iteration ----------------------  +
c     +  7. derived variables                                           +
c     +  8. statistics                                                  +
c     +  9. store analyses and observations                             +
c     + 10. close obs, ana and chk files                                +
c     +     end of the analysis                                         +
c     + A1. write format                                                +
c     +-----------------------------------------------------------------+
      
c 1   ============================================ variable declaration =
      implicit none
c 1.1 ------------------------------------------------------- dimension -
      integer mobs,npar,idim,jdim,nhor,nho2
c 1.2 -------------------------------------------------------- namelist -
      integer   iy,im,id,ih
      integer   maxiter(npar)
      real      latm,lonm,dlat,dlon,rmax,alpha(npar)
      real      gscale(npar),sdevfg(npar),sdevob(npar)
      character exper*3
      logical 	lfgs,lref
c 1.3 ------------------------------------------------------------ grid -
      integer iana,jana
      real    lats,lonw,dlatfgs,dlonfgs
      real    ana(idim,jdim),fgs(idim,jdim),amb(idim,jdim)
      real    p_ana(idim,jdim),t_ana(idim,jdim),u_ana(idim,jdim)
      real    v_ana(idim,jdim),d_ana(idim,jdim)
      real    zana(idim,jdim)
      real    grad_o(idim,jdim),grad_b(idim,jdim),grad(idim,jdim)
c 1.4 --------------------------------------- observational data fields -
      integer   nobs,iiiobs(mobs),nrobs(mobs),flgobs(mobs,npar)
      real      oriobs(mobs,npar),latobs(mobs),lonobs(mobs)
      character filnm*12
c 1.5 ------------------------------ observations selected for analysis -
      integer nana,iiival(mobs),nrval(mobs)
      real    latval(mobs),lonval(mobs),valobs(mobs),valmean
      real    valfgs(mobs),valinn(mobs),valana(mobs)
c 1.6 --------------------------------- information about rejected data -
      integer nskip,lskip(mobs),flag(mobs),dev(mobs)
      logical skip(mobs)
c 1.7 ---------------------------------------- covariance matrix fields -
      real bi(idim*jdim*idim*jdim)
c 1.8  -------------------------------------- administrative parameters -
      real    lat,lon,cost,jo,jb
      integer i,j,iter
      integer ipar,ios
      integer lunnam,lunobs,lunfgs,lunana,lunref
      real    clat ! cos(lat)=111111.1111
c 2   ============================================== derived parameters =
      nhor=idim*jdim
c 3   ===================================== write some input parameters =
      write(6,1001)
      write(6,'(2a)')' exp : ',exper
      write(6,'(a,4(1x,i2.2))')' date:',iy-1900,im,id,ih
      write(6,1001)
      write(6,*)' parameter              pressure       temperature'
      write(6,1002)
      write(6,*)' sdev of fg          ',sdevfg
      write(6,*)' sdev of obs         ',sdevob
      write(6,*)' Gaussian parameters ',gscale
      write(6,1001)
c 4   ===================================== open input and output files =
c 4.1 ------------------------------------------------ observation file -
      write(filnm(1:12),'(4i2.2,a4)') mod(iy,100),im,id,ih,'.obs'
      open(lunobs,file=filnm,iostat=ios,status='old')
c 4.3 ------------------------------------------------- firstguess file -
      if(lfgs) then
         write(filnm(1:12),'(4i2.2,a4)') mod(iy,100),im,id,ih,'.fgs'
         open(lunfgs,file=filnm,iostat=ios,status='old')
         read(lunfgs,*) iana,jana,lats,lonw,dlatfgs,dlonfgs
         if(dlatfgs.ne.dlat.or.dlonfgs.ne.dlon) then
            write(6,*) 'from fgs, dlon and dlat =',dlonfgs,dlatfgs
            write(6,*) 'namelist, dlon and dlat =',dlon,dlat
            stop
         endif
      endif
c 4.4 ----------------------------------------------------- result file -
      write(filnm(1:12),'(4i2.2,a,a3)')mod(iy,100),im,id,ih,'.',exper
      open(lunana,file=filnm,iostat=ios,status='unknown')
c 5   ============================= read and (gross) check observations =
      call getobs(lunobs,npar,mobs,nobs,
     +            iiiobs,nrobs,latobs,lonobs,oriobs,flgobs)
c 6   =============================== analysis loop over all parameters =
      do ipar=1,2
c 6.1    ---------------------------------------------- read firstguess -
         if(lfgs) call getfld(idim,jdim,fgs,lunfgs)
c 6.2    ------------------------------------------------- VAR analysis -
c 6.2.1     ........................... select influencing observations .
            call choose_box(1,idim,jdim,0,0,
     +            iiiobs,latobs,lonobs,oriobs(1,ipar),
     +            nrobs,nobs,latm,lonm,dlat,dlon,rmax,mobs,1,
     +            iiival,nrval,latval,lonval,valobs,nana)
	    write(6,*) 'selected obs : ',nana
c 6.2.2     ................... construct and convert covariance matrix .
	    call binv(idim,jdim,bi,latm,lonm,dlat,dlon,clat,
     +                sdevfg(ipar),gscale(ipar))
c 6.2.3     ........................... use fgs as ana before at iter=0 .
	    write(6,*)' ana=fgs '
	    if(.not.lfgs) 
     1       call avefgs(idim*jdim,fgs,mobs,npar,oriobs,nrobs,ipar)
	    call hcpa2b(idim*jdim,fgs,ana)
c           .......................... loop for minimization iterations .
            do iter = 1,maxiter(ipar)
c 6.2.4        ............ transform ana to observation locations (Hx) .
	       call H(idim,jdim,nana,latval,lonval,
     +                latm,lonm,dlat,dlon,ana,valana)
c 6.2.5        .......................... calculation innovation vector .
               do i=1,nana
		  valinn(i) = valana(i)-valobs(i)
	       enddo
c 6.2.6        ........ cost function gradient for observation grad(Jo) .
               call hab2c(1,nana,1,valinn,valinn,jo)
	       jo = jo /(2*sdevob(ipar)*sdevob(ipar))
               do i=1,nana
		  valinn(i) = valinn(i) /(sdevob(ipar)*sdevob(ipar))
	       enddo
	       call hseta2c(idim*jdim,grad_o,0.)
	       call H_ad(idim,jdim,nana,latval,lonval,
     +                latm,lonm,dlat,dlon,grad_o,valinn)
c 6.2.7        ......... cost function gradient for background grad(Jb) .
	       do j=1,jdim
	       do i=1,idim
		  amb(i,j) = ana(i,j) - fgs(i,j)
	       enddo
	       enddo
               call hab2c(nhor,nhor,1,bi,amb,grad_b)
               call hab2c(1,nhor,1,amb,grad_b,jb)
	       jb = jb/2.
c 6.2.8        .......... cost function gradient grad=grad(Jo)+grad(Jb) .
               do i=1,idim
	       do j=1,jdim
		  grad(i,j)=grad_b(i,j) + grad_o(i,j)
	       enddo
	       enddo
c 6.2.9        ...................................... minimization step .
               call stemin(nhor,ana,grad,alpha(ipar))
	       write(6,'(a,i5,a,3e12.5)')
     +              'iter = ',iter, '  jb,jo,j= ',jb,jo,jb+jo
c           ................................. end of loop for iteration .
	    enddo
	    if(ipar.eq.1) then
	       call hcpa2b(idim*jdim,ana,p_ana)
	    else
	       call hcpa2b(idim*jdim,ana,t_ana)
	    endif
      enddo
c 7   ============================================== derived variables =
      call derivf(t_ana,p_ana,
     +            u_ana,v_ana,d_ana,idim,jdim,latm,clat,dlat)
c 8   ===================================================== statistics =
	 write(6,*)' statistics :'
	 call statis('   p min/max = ',idim,jdim,p_ana)
	 call statis('   t min/max = ',idim,jdim,t_ana)
	 call statis('   u min/max = ',idim,jdim,u_ana)
	 call statis('   v min/max = ',idim,jdim,v_ana)
	 call statis(' dyn min/max = ',idim,jdim,d_ana)
      write(6,1001)
c 9   =========================== store the analyses and observations =
      write(lunana,'(2i4,4f10.4)')idim,jdim,
     +              latm-jdim/2*dlat,lonm-idim/2*dlon,dlat,dlon
      call putfld('p',lunana,idim,jdim,p_ana,idim,jdim)
      call putfld('t',lunana,idim,jdim,t_ana,idim,jdim)
cc    call putfld('u',lunana,idim,jdim,u_ana,idim,jdim)
cc    call putfld('v',lunana,idim,jdim,v_ana,idim,jdim)
      call putobs('obs',mobs,npar,nobs,lunana,
     x		iiiobs,latobs,lonobs,oriobs,flgobs)
c 10  ================================== close obs, ana and chk files =
      close(lunobs)
      close(lunfgs)
      close(lunana)
c     =========================================== end of the analysis =
      write(6,*)'analysis finished without any error!'
      write(6,1001)
      return
c A1  ======================================================= formats =
1001  format("=======================================================")
1002  format("-------------------------------------------------------")
      end
