      program analysis

c     +---------------------------------------------------------------------+
c     + A numerical lab for data analysis  (Xiang-Yu Huang, 980801)         +
c     +  1. namelist							    +
c     +     1.1 definition                                                  +
c     +     1.2 default values                                              +
c     +     1.3 read values                                                 +
c     +  2. analysis                                                        +
c     +     2.1 VAR : VARiational analysis                                  +
c     +     2.2 VAN : Variational Analysis with No inbersion of B           +
c     +     2.3 VAF : Variational Analysis using a Filter                   +
c     +     2.4 OI  : Optimum interpolation (BLUE with default parameters)  +
c     +     2.5 PSAS: Physical-space Statistical Analysis System            +
c     +---------------------------------------------------------------------+

c 1.  namelist
c 1.1 definition
      integer   mobs,npar,idim,jdim,nmax,nmin
      parameter (npar=2)
      integer   ischeme
      integer   maxiter(npar),igcut,jgcut
      real      alpha(npar)
      integer   nbox,iy,im,id,ih
      real      latm,lonm,dlat,dlon,rmax
      real      gscale(npar),sdevfg(npar),sdevob(npar)
      real      tolera1(npar),tolera2(npar)
      character exper*3
      logical 	lfgs,lref
      logical 	lwin
      logical 	lplot
      namelist  /params/ischeme,maxiter,alpha,
     +                  mobs,idim,jdim,nmax,nmin,
     +                  nbox,iy,im,id,ih,
     +                  igcut,jgcut,lwin,
     +                  latm,lonm,dlat,dlon,
     +                  gscale,rmax,sdevfg,sdevob,tolera1,tolera2,
     +  	        exper,lfgs,lref,lplot
c 1.1 default values
      data ischeme/1/ 			! ischeme 1-VAR 2-VAN 3-VAF 4-OI 5-PSAS
      data exper/'E01'/ 		! experiment name
      data iy,im,id,ih/1992,3,3,3/ 	! time
      data mobs/1100/			! maximum observations
      data idim,jdim/21,21/ 		! analysis grid dimensions
      data gscale/2*200000./   		! covariance scale 
      data rmax /4500000./		! dat selection radius
      data sdevfg/1.,1./		! standard dev of fgs 
      data sdevob/1.,1./		! standard dev of obs
      data lunobs/16/   		! obs file unit
      data lunfgs/17/   		! fgs file unit
      data lunana/18/   		! ana file unit
      data lunref/19/   		! ref file unit
      data latm,lonm/56.5,14./ 		! grid center coordinate
      data dlat,dlon/0.3,0.6/  		! grid resolution
      data clat/111111.1111/		! r sin(pi)
      data lfgs /.true./		! read in firstguess
      data lref /.false./  		! read in firstguess
c VAR,VAN,VAF,PSAS
      data alpha/0.01,0.01/		! mini step size
      data maxiter/10,10/		! max iter number
c VAF
      data lwin /.true./ 		! use the Lanczos window
      data igcut,jgcut/4,4/		! filter span (*2=filter order)
c OI
      data nbox/1/			! OI boxes
      data nmax,nmin/1100,10/  		! max and min obs in each OI box
      data tolera1/100.,100./ 		! error bars for the GROSS check
      data tolera2/100.,100./ 		! error bars for the OI check
c 1.3 read new values
         open(11,file='params.dat',status='old')
         read(11,params)
         close(11)
c 2.  analysis
c 2.1 VAR : VARiational analysis
         if(ischeme.eq.1)
     1      call var(mobs,npar,iy,im,id,ih,exper,
     2           idim,jdim,gscale,rmax,sdevfg,sdevob,
     3           lunobs,lunfgs,lunana,lunref,
     4           latm,lonm,dlat,dlon,clat,
     5           lfgs,lref,maxiter,alpha)
c 2.2 VAN : Variational Analysis with No inbersion of B
         if(ischeme.eq.2)
     1      call van(mobs,npar,iy,im,id,ih,exper,
     2           idim,jdim,gscale,rmax,sdevfg,sdevob,
     3           lunobs,lunfgs,lunana,lunref,
     4           latm,lonm,dlat,dlon,clat,
     5           lfgs,lref,maxiter,alpha)
c 2.3 VAF : Variational Analysis using a Filter
         if(ischeme.eq.3)
     1      call vaf(mobs,npar,iy,im,id,ih,exper,
     2           idim,jdim,gscale,rmax,sdevfg,sdevob,
     3           lunobs,lunfgs,lunana,lunref,
     4           latm,lonm,dlat,dlon,clat,
     5           lfgs,lref,maxiter,alpha,
     6           igcut,jgcut,lwin)
c 2.4 OI  : Optimum interpolation (BLUE with default parameters)
         if(ischeme.eq.4)
     1      call oi(mobs,nbox,nmax,nmin,npar,iy,im,id,ih,exper,
     2           idim,jdim,gscale,rmax,sdevfg,sdevob,
     3           tolera1,tolera2,
     4           lunobs,lunfgs,lunana,
     5           latm,lonm,dlat,dlon,clat,lfgs)
c 2.5 PSAS: Physical-space Statistical Analysis System
         if(ischeme.eq.5)
     1      call psas(mobs,npar,iy,im,id,ih,exper,
     2           idim,jdim,gscale,rmax,sdevfg,sdevob,
     3           lunobs,lunfgs,lunana,lunref,
     4           latm,lonm,dlat,dlon,clat,
     5           lfgs,lref,maxiter,alpha)
      if(lplot) call system('./map')
      end
